@@ -14,76 +14,40 @@ def _iter_module_files():
     loaded files from modules, all files in folders of already loaded modules
     as well as all files reachable through a package.
     """
-    found = set()
-    entered = set()
-
-    def _verify_file(filename):
-        if not filename:
-            return
-        filename = os.path.abspath(filename)
-        old = None
-        while not os.path.isfile(filename):
-            old = filename
-            filename = os.path.dirname(filename)
-            if filename == old:
-                break
-        else:
-            if filename[-4:] in ('.pyc', '.pyo'):
-                filename = filename[:-1]
-            if filename not in found:
-                found.add(filename)
-                return filename
-
-    def _recursive_walk(path_entry):
-        if path_entry in entered:
-            return
-        entered.add(path_entry)
-        try:
-            for filename in os.listdir(path_entry):
-                path = os.path.join(path_entry, filename)
-                if os.path.isdir(path):
-                    for filename in _recursive_walk(path):
-                        yield filename
-                else:
-                    if not filename.endswith(('.py', '.pyc', '.pyo')):
-                        continue
-                    filename = _verify_file(path)
-                    if filename:
-                        yield filename
-        except OSError:
-            pass
-
     # The list call is necessary on Python 3 in case the module
     # dictionary modifies during iteration.
-    for path_entry in list(sys.path):
-        for filename in _recursive_walk(os.path.abspath(path_entry)):
-            yield filename
-
     for module in list(sys.modules.values()):
         if module is None:
             continue
-        filename = _verify_file(getattr(module, '__file__', None))
+        filename = getattr(module, '__file__', None)
         if filename:
-            yield filename
-            for filename in _recursive_walk(os.path.dirname(filename)):
-                yield filename
-        for package_path in getattr(module, '__path__', ()):
-            for filename in _recursive_walk(os.path.abspath(package_path)):
+            old = None
+            while not os.path.isfile(filename):
+                old = filename
+                filename = os.path.dirname(filename)
+                if filename == old:
+                    break
+            else:
+                if filename[-4:] in ('.pyc', '.pyo'):
+                    filename = filename[:-1]
                 yield filename
 
 
 def _find_observable_paths(extra_files=None):
     """Finds all paths that should be observed."""
     rv = set(os.path.abspath(x) for x in sys.path)
+
     for filename in extra_files or ():
-        rv.append(os.path.dirname(os.path.abspath(filename)))
+        rv.add(os.path.dirname(os.path.abspath(filename)))
+
     for module in list(sys.modules.values()):
         fn = getattr(module, '__file__', None)
         if fn is None:
             continue
         fn = os.path.abspath(fn)
         rv.add(os.path.dirname(fn))
-    return rv
+
+    return _find_common_roots(rv)
 
 
 def _find_common_roots(paths):
@@ -97,6 +61,7 @@ def _find_common_roots(paths):
         node.clear()
 
     rv = set()
+
     def _walk(node, path):
         for prefix, child in iteritems(node):
             _walk(child, path + (prefix,))
@@ -140,14 +105,18 @@ class ReloaderLoop(object):
                     if isinstance(value, text_type):
                         new_environ[key] = value.encode('iso-8859-1')
 
-            exit_code = subprocess.call(args, env=new_environ)
+            exit_code = subprocess.call(args, env=new_environ,
+                                        close_fds=False)
             if exit_code != 3:
                 return exit_code
 
     def trigger_reload(self, filename):
+        self.log_reload(filename)
+        sys.exit(3)
+
+    def log_reload(self, filename):
         filename = os.path.abspath(filename)
         _log('info', ' * Detected change in %r, reloading' % filename)
-        sys.exit(3)
 
 
 class StatReloaderLoop(ReloaderLoop):
@@ -156,7 +125,8 @@ class StatReloaderLoop(ReloaderLoop):
     def run(self):
         mtimes = {}
         while 1:
-            for filename in chain(_iter_module_files(), self.extra_files):
+            for filename in chain(_iter_module_files(),
+                                  self.extra_files):
                 try:
                     mtime = os.stat(filename).st_mtime
                 except OSError:
@@ -190,11 +160,20 @@ class WatchdogReloaderLoop(ReloaderLoop):
                     self.trigger_reload(filename)
 
         class _CustomHandler(FileSystemEventHandler):
+
             def on_created(self, event):
                 _check_modification(event.src_path)
+
             def on_modified(self, event):
                 _check_modification(event.src_path)
 
+            def on_moved(self, event):
+                _check_modification(event.src_path)
+                _check_modification(event.dest_path)
+
+            def on_deleted(self, event):
+                _check_modification(event.src_path)
+
         reloader_name = Observer.__name__.lower()
         if reloader_name.endswith('observer'):
             reloader_name = reloader_name[:-8]
@@ -207,10 +186,11 @@ class WatchdogReloaderLoop(ReloaderLoop):
         self.should_reload = False
 
     def trigger_reload(self, filename):
-        # This is called inside an event handler, which means we can't throw
-        # SystemExit here. https://github.com/gorakhargosh/watchdog/issues/294
+        # This is called inside an event handler, which means throwing
+        # SystemExit has no effect.
+        # https://github.com/gorakhargosh/watchdog/issues/294
         self.should_reload = True
-        ReloaderLoop.trigger_reload(self, filename)
+        self.log_reload(filename)
 
     def run(self):
         watches = {}
@@ -219,17 +199,22 @@ class WatchdogReloaderLoop(ReloaderLoop):
 
         while not self.should_reload:
             to_delete = set(watches)
-            paths = _find_common_roots(
-                _find_observable_paths(self.extra_files))
+            paths = _find_observable_paths(self.extra_files)
             for path in paths:
                 if path not in watches:
                     try:
                         watches[path] = observer.schedule(
                             self.event_handler, path, recursive=True)
-                    except OSError:
-                        # "Path is not a directory". We could filter out
-                        # those paths beforehand, but that would cause
-                        # additional stat calls.
+                    except OSError as e:
+                        message = str(e)
+
+                        if message != "Path is not a directory":
+                            # Log the exception
+                            _log('error', message)
+
+                        # Clear this path from list of watches We don't want
+                        # the same error message showing again in the next
+                        # iteration.
                         watches[path] = None
                 to_delete.discard(path)
             for path in to_delete:
