@@ -10,7 +10,6 @@
 """
 import re
 import os
-import sys
 import posixpath
 import mimetypes
 from itertools import chain
@@ -20,11 +19,12 @@ from datetime import datetime
 from functools import partial, update_wrapper
 
 from werkzeug._compat import iteritems, text_type, string_types, \
-     implements_iterator, make_literal_wrapper, to_unicode, to_bytes, \
-     wsgi_get_bytes, try_coerce_native, PY2
+    implements_iterator, make_literal_wrapper, to_unicode, to_bytes, \
+    wsgi_get_bytes, try_coerce_native, PY2
 from werkzeug._internal import _empty_stream, _encode_idna
 from werkzeug.http import is_resource_modified, http_date
 from werkzeug.urls import uri_to_iri, url_quote, url_parse, url_join
+from werkzeug.filesystem import get_filesystem_encoding
 
 
 def responder(f):
@@ -42,8 +42,8 @@ def responder(f):
 
 def get_current_url(environ, root_only=False, strip_querystring=False,
                     host_only=False, trusted_hosts=None):
-    """A handy helper function that recreates the full URL for the current
-    request or parts of it.  Here an example:
+    """A handy helper function that recreates the full URL as IRI for the
+    current request or parts of it.  Here an example:
 
     >>> from werkzeug.test import create_environ
     >>> env = create_environ("/?param=foo", "http://localhost/script")
@@ -60,6 +60,15 @@ def get_current_url(environ, root_only=False, strip_querystring=False,
     If the host is not in there it will raise a
     :exc:`~werkzeug.exceptions.SecurityError`.
 
+    Note that the string returned might contain unicode characters as the
+    representation is an IRI not an URI.  If you need an ASCII only
+    representation you can use the :func:`~werkzeug.urls.iri_to_uri`
+    function:
+
+    >>> from werkzeug.urls import iri_to_uri
+    >>> iri_to_uri(get_current_url(env))
+    'http://localhost/script/?param=foo'
+
     :param environ: the WSGI environment to get the current URL from.
     :param root_only: set `True` if you only want the root URL.
     :param strip_querystring: set to `True` if you don't want the querystring.
@@ -120,17 +129,20 @@ def host_is_trusted(hostname, trusted_list):
 
 
 def get_host(environ, trusted_hosts=None):
-    """Return the real host for the given WSGI environment.  This takes care
-    of the `X-Forwarded-Host` header.  Optionally it verifies that the host
-    is in a list of trusted hosts.  If the host is not in there it will raise
-    a :exc:`~werkzeug.exceptions.SecurityError`.
+    """Return the real host for the given WSGI environment.  This first checks
+    the `X-Forwarded-Host` header, then the normal `Host` header, and finally
+    the `SERVER_NAME` environment variable (using the first one it finds).
+
+    Optionally it verifies that the host is in a list of trusted hosts.
+    If the host is not in there it will raise a
+    :exc:`~werkzeug.exceptions.SecurityError`.
 
     :param environ: the WSGI environment to get the host of.
     :param trusted_hosts: a list of trusted hosts, see :func:`host_is_trusted`
                           for more information.
     """
     if 'HTTP_X_FORWARDED_HOST' in environ:
-        rv = environ['HTTP_X_FORWARDED_HOST'].split(',')[0].strip()
+        rv = environ['HTTP_X_FORWARDED_HOST'].split(',', 1)[0].strip()
     elif 'HTTP_HOST' in environ:
         rv = environ['HTTP_HOST']
     else:
@@ -170,7 +182,7 @@ def get_input_stream(environ, safe_fallback=True):
     .. versionadded:: 0.9
 
     :param environ: the WSGI environ to fetch the stream from.
-    :param safe: indicates weather the function should use an empty
+    :param safe: indicates whether the function should use an empty
                  stream as safe fallback or just return the original
                  WSGI input stream if it can't wrap it safely.  The
                  default is to return an empty string in those cases.
@@ -179,7 +191,7 @@ def get_input_stream(environ, safe_fallback=True):
     content_length = get_content_length(environ)
 
     # A wsgi extension that tells us if the input is terminated.  In
-    # that case we return the stream unchanged as we know we can savely
+    # that case we return the stream unchanged as we know we can safely
     # read it until the end.
     if environ.get('wsgi.input_terminated'):
         return stream
@@ -414,6 +426,7 @@ def extract_path_info(environ_or_baseurl, path_or_url, charset='utf-8',
 
 
 class SharedDataMiddleware(object):
+
     """A WSGI middleware that provides static content for development
     environments or simple server setups. Usage is quite simple::
 
@@ -466,7 +479,7 @@ class SharedDataMiddleware(object):
     :param disallow: a list of :func:`~fnmatch.fnmatch` rules.
     :param fallback_mimetype: the fallback mimetype for unknown files.
     :param cache: enable or disable caching headers.
-    :Param cache_timeout: the cache timeout in seconds for the headers.
+    :param cache_timeout: the cache timeout in seconds for the headers.
     """
 
     def __init__(self, app, exports, disallow=None, cache=True,
@@ -510,11 +523,12 @@ class SharedDataMiddleware(object):
 
     def get_package_loader(self, package, package_path):
         from pkg_resources import DefaultProvider, ResourceManager, \
-             get_provider
+            get_provider
         loadtime = datetime.utcnow()
         provider = get_provider(package)
         manager = ResourceManager()
         filesystem_bound = isinstance(provider, DefaultProvider)
+
         def loader(path):
             if path is None:
                 return None, None
@@ -545,7 +559,7 @@ class SharedDataMiddleware(object):
 
     def generate_etag(self, mtime, file_size, real_filename):
         if not isinstance(real_filename, bytes):
-            real_filename = real_filename.encode(sys.getfilesystemencoding())
+            real_filename = real_filename.encode(get_filesystem_encoding())
         return 'wzsdm-%d-%s-%s' % (
             mktime(mtime.timetuple()),
             file_size,
@@ -555,14 +569,14 @@ class SharedDataMiddleware(object):
     def __call__(self, environ, start_response):
         cleaned_path = get_path_info(environ)
         if PY2:
-            cleaned_path = cleaned_path.encode(sys.getfilesystemencoding())
+            cleaned_path = cleaned_path.encode(get_filesystem_encoding())
         # sanitize the path for non unix systems
         cleaned_path = cleaned_path.strip('/')
         for sep in os.sep, os.altsep:
             if sep and sep != '/':
                 cleaned_path = cleaned_path.replace(sep, '/')
-        path = '/'.join([''] + [x for x in cleaned_path.split('/')
-                                if x and x != '..'])
+        path = '/' + '/'.join(x for x in cleaned_path.split('/')
+                              if x and x != '..')
         file_loader = None
         for search_path, loader in iteritems(self.exports):
             if search_path == path:
@@ -608,6 +622,7 @@ class SharedDataMiddleware(object):
 
 
 class DispatcherMiddleware(object):
+
     """Allows one to mount middlewares or applications in a WSGI application.
     This is useful if you want to combine multiple WSGI applications::
 
@@ -628,9 +643,8 @@ class DispatcherMiddleware(object):
             if script in self.mounts:
                 app = self.mounts[script]
                 break
-            items = script.split('/')
-            script = '/'.join(items[:-1])
-            path_info = '/%s%s' % (items[-1], path_info)
+            script, last_item = script.rsplit('/', 1)
+            path_info = '/%s%s' % (last_item, path_info)
         else:
             app = self.mounts.get(script, self.app)
         original_script_name = environ.get('SCRIPT_NAME', '')
@@ -641,6 +655,7 @@ class DispatcherMiddleware(object):
 
 @implements_iterator
 class ClosingIterator(object):
+
     """The WSGI specification requires that all middlewares and gateways
     respect the `close` callback of an iterator.  Because it is useful to add
     another close action to a returned iterator and adding a custom iterator
@@ -707,6 +722,7 @@ def wrap_file(environ, file, buffer_size=8192):
 
 @implements_iterator
 class FileWrapper(object):
+
     """This class can be used to convert a :class:`file`-like object into
     an iterable.  It yields `buffer_size` blocks until the file is fully
     read.
@@ -886,6 +902,7 @@ def make_chunk_iter(stream, separator, limit=None, buffer_size=10 * 1024):
 
 @implements_iterator
 class LimitedStream(object):
+
     """Wraps a stream so that it doesn't read more than n bytes.  If the
     stream is exhausted and the caller tries to get more bytes from it
     :func:`on_exhausted` is called which by default returns an empty
